'''
The MoinMoin Uploads Center

This is a MoinMoin application that helps with managing uploaded files.

> Hey! What do you mean by "uploaded files"?

Uploaded files are *not* attachments. Attachments are bound to a page and are uploaded
by means of HTML and HTTP. However, often it is easier to upload files by means of FTP or
the like, and use a web front-end just to access them one by one.

This is where the Upload Center comes in. It scans the htdocs/_uploads directory and all it's
sub-directories. Whereever it findes a file named "!readme.txt" (note: case matters), it
collects the files inside that directory and shows them in a HTML table.

Go aheead! Try it and see for yourself!

Cheers and Happy Uploading
ZoranIsailovski

----
Copyright (C) 2007 Zoran Isailovski
Licensed under the Apache License, Version 2.0 -- http://www.apache.org/licenses/LICENSE-2.0
'''

#-- configuration ------------------------------------------------------------

UPLOADS_DIR  = r'htdocs\_uploads' % globals() # relative to "wiki" folder
INFOFILENAME = '!readme.txt'

#-----------------------------------------------------------------------------

import sys, os
import tmpl

from cStringIO import StringIO

from MoinMoin import wikiutil
from MoinMoin.parser.wiki import Parser
from MoinMoin.formatter.text_html import Formatter
from MoinMoin.Page import Page

#-----------------------------------------------------------------------------
__filedir = os.path.join(os.path.dirname(__file__))
__moindir = os.path.abspath(os.path.join(__filedir, '../../../..'))
UPLOADS_DIR = os.path.join(__moindir, UPLOADS_DIR)
#-----------------------------------------------------------------------------

def execute(pagename, request):
    _ = request.getText

    # prepare for processing wiki format
    request.formatter = Formatter(request)
    request.formatter.page = Page(request, pagename)

    request.http_headers()
    wikiutil.send_title(request, _('MoinMoin Uploads Center'), pagename=pagename)
    try:
        process(request)
    finally:
        wikiutil.send_footer(request, pagename)

######################################################################
## 
## core
## 
######################################################################

def process(request):
    all = filesInUploadArea()

    BANG = '#!wiki'
    BANG_LEN = len(BANG)
    for each in all:
        s = each.desc
        each.iswiki = iswiki = s.startswith(BANG)
        if iswiki:
            ##s = '\n'.join( s.splitlines()[1:] ) # remove bang line
            s = s[BANG_LEN:].lstrip() # remove bang
            each.desc = formatAsWiki(s, request)

    s = tmpl.render(tmpl.this('main'), filegroups=all, py=__builtins__)
    request.write(s)

class FileGroup:
    def __init__(self, folder, desc, files):
        self.folder = folder.replace(os.sep, '/').decode('Latin')
        self.desc = desc.decode('Latin')
        self.files = [ s.decode('Latin') for s in files ]
        self.title = relpath(folder, UPLOADS_DIR).decode('Latin')

def filesInUploadArea(subdir=''):
    return list(iterFilesInUploadArea(subdir))

def iterFilesInUploadArea(subdir=''):
    d = UPLOADS_DIR
    if subdir: d = os.path.join(d, subdir)
    for (folder, subdirs, files) in os.walk(d):
        desc = folder
        i = indexof(INFOFILENAME, files)
        if i < 0: continue
        del files[i]
        desc = fread(os.path.join(folder, INFOFILENAME)).strip()
        yield FileGroup(folder=folder, desc=desc, files=files)

######################################################################
## 
## aux
## 
######################################################################

def fread(path):
    f = open(path, 'rt')
    try: return f.read()
    finally: f.close()

def indexof(value, seq):
    try: return seq.index(value)
    except ValueError: return -1

def relpath(path, basedir=UPLOADS_DIR):
    pfx = basedir + os.sep
    if path.startswith(pfx): path = path[len(pfx):]
    return path.replace(os.sep, '/')

def formatAsWiki(text, request):
    buf = StringIO()
    request.redirect(buf)
    try:
        ##request.form = request.args = request.setup_args({})
        parser = Parser(text, request)
        parser.format(request.formatter)
        return buf.getvalue()
    finally:
        request.redirect()

